class ObjCType
  attr_accessor :varname
  def initialize( name )
    @varname = name
  end
  def property_definition
    "-subcalss-"
  end
  def default_value
    "-subcalss-"
  end
  def conversion_value origin
    "-subcalss-"
  end
  def to_dictionary_item
    "-subcalss-"
  end
  def inEquality_test other
    "-subcalss-"
  end
  def description_row
    "-subcalss-"
  end
  def copyrow newVarName
    "-subcalss-"
  end
  def hash_row
    "-subcalss-"
  end
end

class TypeBool  < ObjCType
  attr_accessor :varname
  def property_definition
    "@property (nonatomic) BOOL #{@varname};"
  end
  def default_value
    "NO"
  end

  def conversion_value origin
    "[#{origin} boolValue]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : @(self.#{@varname})  ?: @(YES)"
  end
  def inEquality_test other
    "self.#{self.varname}  != #{other}.#{self.varname}"
  end
  def description_row
    "@\"self.#{self.varname} = %@\" , self.#{self.varname} ? @\"YES\" : @\"NO\""
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = self.#{self.varname};"
  end
  def hash_row
    "(self.#{self.varname}?1231:1237);"
  end

end

class TypeInt  < ObjCType
  attr_accessor :varname
  def property_definition
    "@property (nonatomic) NSInteger #{@varname};"
  end
  def default_value
    "0"
  end
  def conversion_value origin
    "[#{origin} integerValue]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : @(self.#{@varname})  ?: @(0)"
  end
  def inEquality_test other
    "self.#{self.varname}  != #{other}.#{self.varname}"
  end
  def description_row
    "@\"self.#{self.varname} = %zd\" , self.#{self.varname}"
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = self.#{self.varname};"
  end
  def hash_row
    "self.#{self.varname};"
  end
end

class TypeFloat  < ObjCType
  attr_accessor :varname
  def property_definition
    "@property (nonatomic) double #{@varname};"
  end
  def default_value
    "0."
  end
  def conversion_value origin
    "[#{origin} doubleValue]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : @(self.#{@varname})  ?: @(0.)"
  end
  def inEquality_test other
    "self.#{self.varname}  != #{other}.#{self.varname}"
  end
  def description_row
    "@\"self.#{self.varname} = %lf\" , self.#{self.varname}"
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = self.#{self.varname};"
  end
  def hash_row
    "self.#{self.varname};" #is rounded
  end
end

class TypeString  < ObjCType
  attr_accessor :varname
  def property_definition
    "@property (nonatomic) NSString *#{@varname};"
  end
  def default_value
    "@\"\""
  end
  def conversion_value origin
    "[#{origin} description]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : self.#{@varname}  ?: @\"\""
  end
  def inEquality_test other
    "![self.#{self.varname}  isEqual:#{other}.#{self.varname}]"
  end
  def description_row
    "@\"self.#{self.varname} = %@\" , self.#{self.varname}"
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = [self.#{self.varname} copy];"
  end
  def hash_row
    "[self.#{self.varname} hash];"
  end
end

class TypeDate  < ObjCType
  attr_accessor :varname
  def property_definition
    "@property (nonatomic) NSDate *#{@varname};"
  end
  def default_value
    "[NSDate dateWithTimeIntervalSince1970:0.]"
  end
  def conversion_value origin
    "[NSDate ic_dateFromIso8601String:#{origin}]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : [self.#{@varname} ic_iso8601String]  ?: @\"\""
  end
  def inEquality_test other
    "![self.#{self.varname}  isEqualToDate:#{other}.#{self.varname}]"
  end
  def description_row
    "@\"self.#{self.varname} = %@\" , [self.#{self.varname} ic_iso8601String]"
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = [self.#{self.varname} copy];"
  end
  def hash_row
    "[self.#{self.varname} hash];"
  end
end



class TypeCustomObject  < ObjCType
  attr_accessor :varname
  attr_accessor :var_type
  def initialize( name , type)
    super(name)
    @var_type = type
  end
  def property_definition
    "@property (nonatomic) #{@var_type} *#{@varname};"
  end
  def default_value
    "[#{@var_type} new]"
  end
  def conversion_value origin
    # "[#{origin} description]"
    "[[#{var_type} alloc] initWithDict:#{origin}]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : [self.#{@varname} toDict]  ?: @{}"
  end
  def inEquality_test other
    "![self.#{self.varname}  isEqual:#{other}.#{self.varname}]"
  end
  def description_row
    "@\"self.#{self.varname} = %@\" , self.#{self.varname}"
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = [self.#{self.varname} copyWithZone:nil];"
  end
  def hash_row
    "[self.#{self.varname} hash];"
  end
end


###############################################################################
##                           A R R A Y                                       ##
###############################################################################

class TypeArray  < ObjCType
  attr_accessor :varname
  attr_accessor :var_sub_type
  def initialize( name , sub_type)
    super(name)

    @var_sub_type = sub_type
  end

  def property_definition
    "@property (nonatomic) NSArray *#{@varname}; //@[#{var_sub_type}]"
  end
  def default_value
    "@[]"
  end
  def conversion_value origin
    "[NSArray arrayWithArray:#{origin}]"
  end
  def to_dictionary_item
    "@\"#{@varname}\" : self.#{@varname}  ?: @[]"
  end
  def inEquality_test other
    "![self.#{self.varname}  isEqual:#{other}.#{self.varname}]"
  end
  def description_row
    "@\"self.#{self.varname} = %@\" , self.#{self.varname}"
  end
  def copyrow newVarName
    "#{newVarName}.#{self.varname}  = [self.#{self.varname} copyWithZone:nil];"
  end
  def hash_row
    "[self.#{self.varname} hash];"
  end
end
